/*
 * Created on 2005/01/23
 *
 */
package jnet.application.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Insets;
import java.util.ArrayList;

import javax.swing.JPanel;

import jnet.application.util.ViewerUtilities;
import jnet.gps.geo.DistanceCalculater;
import jnet.hiromicam.buff.FirstLevelInfoBuff;
import jnet.hiromicam.buff.MyTerminal;
import jnet.hiromicam.buff.Terminal;
import jnet.hiromicam.msg.FirstLevelInfo;

/**
 * This class dsplays relative of position between your and other.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class Viewer extends JPanel {
	
	private static Viewer viewer = new Viewer();
	
	/**
	 * size of window.
	 */
	public  static final double WINDOW_SIZE = 240;
	
	/**
	 * This reprecents View Mode.
	 */
	private int viewMode = ViewerUtilities.NORMAL_VIEW;
	
	/**
	 * size of radius of circle.
	 */
	private double radiusSize = 50;
	
	/**
	 * size of target.
	 */
	private int targetSize = 8;
	
	/**
	 * This is live of terminal (ms)
	 */
	private static final int TERMINAL_LIVE = 900000;
	
	/**
	 * This is flag which dysplay target description or not.
	 */
	private boolean targetDescription = true;
	
	/**
	 * This is img valiable.
	 */
	private Image img;
	
	/**
	 * This is file name of background imagge of GPS Viewer
	 */
	private String backgroundImg;
	
	/**
	 * This is flag that draw image or not.
	 * ture:use image, false:don't use image
	 */
	private boolean backgroundImgFlag = false;
	
	/**
	 * This is flag that draw building or not.
	 * true:show building, false:don't show building
	 */
	private boolean buldingFlag = true;
	
	/**
	 * This is flag that draw direction or not.
	 * true:show direction, false;don't show direction
	 */
	private boolean directionFlag = true;	
	
	/**
	 * This is absolutely latitude.
	 */
	private String rockedLatitude = "0";
	
	/**
	 * This is absolutely lontitude.
	 */
	private String rockedLongitude = "0";
	
	/**
	 * This is a default constractor.
	 *
	 */
	public Viewer() {
		setSize((int)WINDOW_SIZE, (int)WINDOW_SIZE);
		this.setBackground(Color.BLACK);
	}
	
	// This method override javadoc of super class.
	public void paint(Graphics g) {
		super.paint(g);
		
		// set x, y on the origin 
		Insets is = getInsets();
		int x = (getWidth() - is.left - is.right ) / 2 + is.left;
		int y = getHeight() / 2;
		
		// The rate of conversion for changing into the coordinates on application from real data.
		double convertX = ((int)WINDOW_SIZE - is.left - is.right) / radiusSize / 2;		// rate of transfor to pxcel from real value.
		double convertY = ((int)WINDOW_SIZE - is.top - is.bottom) / radiusSize / 2;		// rate of transfor to pxcel from real value.
		
		// draw circle
		g.setColor(Color.green);			// set circle collor
		g.drawArc(is.left + (int)WINDOW_SIZE / 4, is.top + (int)WINDOW_SIZE / 4,
				  (int)WINDOW_SIZE / 2, (int)WINDOW_SIZE / 2, 0, 360);
		g.drawArc(is.left, is.top, (int)WINDOW_SIZE, (int)WINDOW_SIZE, 0, 360);
		
		// draw lline
		g.drawLine(0, y, getWidth(), y);				// draw x line
		g.drawLine(x, 0, x, getHeight());	// draw y line
		
		// draw scale of circle
		g.setColor(Color.white);
		Font fnt = new Font("scale", Font.PLAIN, 12);
		g.setFont(fnt);
		FontMetrics fm = getFontMetrics(g.getFont());
		g.drawString(String.valueOf((int)(radiusSize / 2)),
					 x + getWidth() / 4 - is.left - fm.stringWidth(String.valueOf((int)(radiusSize / 2))),
					 y - is.bottom);
		g.drawString(String.valueOf((int)radiusSize), 
					 getWidth() - is.left - fm.stringWidth(String.valueOf((int)radiusSize)),
					 y - is.bottom);
		
		// draw direction
		g.drawString("North", x - is.left - fm.stringWidth("North") / 2, is.top + fm.getHeight());
		g.drawString("East", getWidth() - is.left - is.right - fm.stringWidth("East"), y + fm.getHeight());
		
		// draw terminal.
		if (viewMode == ViewerUtilities.NORMAL_VIEW) {
			// normal view mode.
			
			try {
				// getting my information
				ArrayList list = MyTerminal.getInstance().getFirstLevelInfoLists();
				FirstLevelInfo myMsg = (FirstLevelInfo)list.get(list.size() - 1);
				
				// getting target information.
				ArrayList targetList = FirstLevelInfoBuff.getInstance().getTerminalLists();
				
				for (int i = 0; i < targetList.size(); i++) {
					
					// change color.
					g.setColor(Color.YELLOW);
					
					// get msg list.
					ArrayList targetMsgList = ((Terminal)targetList.get(i)).getFirstLevelInfoLists();
					FirstLevelInfo targetMsg = (FirstLevelInfo)targetMsgList.get(targetMsgList.size() - 1);
					
				 	// x position of terget.
				 	int targetX = x - targetSize / 2
	 						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(myMsg.getLatitude(),
	 																					  myMsg.getLongitude(),
																						  targetMsg.getLatitude(),
																						  targetMsg.getLongitude()));
				 	// y position of terget.
				 	int targetY = y - targetSize / 2
				  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(myMsg.getLatitude(),
				  																		myMsg.getLongitude(),
				  																		targetMsg.getLatitude(),
				  																		targetMsg.getLongitude()));
				 	
				 	// draw
				 	g.fillArc(targetX, targetY, targetSize, targetSize, 0, 360);
				}
			} catch (ArrayIndexOutOfBoundsException e) {
//				e.printStackTrace();
			}
			
		} else if (viewMode == ViewerUtilities.ROCK_VIEW) {
			
		} else if (viewMode == ViewerUtilities.OWN_VIEW) {
			// own view mode.
			
			ArrayList list = MyTerminal.getInstance().getFirstLevelInfoLists();
			FirstLevelInfo targetMsg;
			
			for (int i = list.size() - 1; i >= 0; i++) {
				targetMsg = (FirstLevelInfo)list.get(i);
				
			 	// x position of terget.
			 	int targetX = x - targetSize / 2
 						- (int)(convertX * DistanceCalculater.getHorizontalCordinate(rockedLatitude,
 																					  rockedLongitude,
																					  targetMsg.getLatitude(),
																					  targetMsg.getLongitude()));
			 	// y position of terget.
			 	int targetY = y - targetSize / 2
			  			+ (int)(convertY * DistanceCalculater.getVerticalCordinate(rockedLatitude,
			  																		rockedLongitude,
			  																		targetMsg.getLatitude(),
			  																		targetMsg.getLongitude()));
			 	
			 	// draw
			 	g.fillArc(targetX, targetY, targetSize, targetSize, 0, 360);
			}
		}
	}
	
	/**
	 * This method returns instance of Viewre.
	 * 
	 * @return instance of Viewer.
	 */
	public static Viewer getInstance() {
		return viewer;
	}
	
	/**
	 * This method is for setting radius size of circle.
	 * 
	 * @param radiusSize The radiusSize to set.
	 */
	public void setRadiusSize(double radiusSize) {
		this.radiusSize = radiusSize;
	}
	
	/**
	 * This method is for setting target size.
	 * 
	 * @param targetSize The targetSize to set.
	 */
	public void setTargetSize(int targetSize) {
		this.targetSize = targetSize;
	}
	
	/**
	 * This method is for settin view mode.
	 * 
	 * @param viewMode The viewMode to set.
	 */
	public void setViewMode(int viewMode) {
		this.viewMode = viewMode;
		if (viewMode == ViewerUtilities.ROCK_VIEW) {
			ArrayList list = MyTerminal.getInstance().getFirstLevelInfoLists();
			FirstLevelInfo msg = (FirstLevelInfo)list.get(list.size() - 1);
			rockedLatitude = msg.getLatitude();
			rockedLongitude = msg.getLongitude();
		}
	}
	
	/**
	 * This method returns viewe mode.
	 * 
	 * @return Returns the viewMode.
	 */
	public int getViewMode() {
		return viewMode;
	}
}
