/*
 * Created on 2005/01/23
 *
 */
package jnet.application.gui;

import javax.swing.JPanel;

import javax.swing.JButton;
import java.awt.GridLayout;
import javax.swing.JComboBox;

import jnet.application.util.ViewerUtilities;
import jnet.hiromicam.comm.SerialPortAddapter;
import jnet.hiromicam.core.MsgReceiver;
import jnet.hiromicam.properity.SystemProperty;
/**
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class ViewerControlPanel extends JPanel {
	
	/**
	 * size of window
	 */
	public  static final double WINDOW_HEIGHT = 40;
	
	/**
	 * size of window
	 */
	public  static final double WINDOW_WIDTH = 250;
	
	private String[] viewMode = {"Normal",
								  "Rock",
								  "Own"
	};
	
	private MsgReceiver msgReceiver = null;
	
	private SerialPortAddapter serialPortAddapter = null;
	
	private JButton startButton = null;
	private JButton vieweModeButton = null;
	private JButton stopButton = null;
	private JComboBox jComboBox = null;
	private JPanel jPanel = null;
	private JPanel jPanel1 = null;
	/**
	 * This is the default constructor
	 */
	public ViewerControlPanel() {
		super();
		initialize();
	}
	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private  void initialize() {
		GridLayout gridLayout2 = new GridLayout();
		this.setLayout(gridLayout2);
		this.setSize(250,50);
		gridLayout2.setRows(1);
		gridLayout2.setColumns(2);
		this.add(getJPanel(), null);
		this.add(getJPanel1(), null);
		stopButton.setEnabled(false);
	}
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getStartButton() {
		if (startButton == null) {
			startButton = new JButton();
			startButton.setText("Start");
			startButton.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12));
			startButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					// start Hiromicam Network.
					// The acquisition start of the GPS information from a satellite
					serialPortAddapter = new SerialPortAddapter(SystemProperty.getInstance().getSerialPortNumber());
					serialPortAddapter.start();
					
					// The reception start which is information from other terminals.
					msgReceiver = new MsgReceiver();
					msgReceiver.start();
					
					// unenabled start button
					startButton.setEnabled(false);
					//enabled stop button
					stopButton.setEnabled(true);
				}
			});
		}
		return startButton;
	}
	/**
	 * This method initializes jButton1	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getVieweModeButton() {
		if (vieweModeButton == null) {
			vieweModeButton = new JButton();
			vieweModeButton.setText("Update");
			vieweModeButton.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12));
			vieweModeButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					Viewer.getInstance().repaint();
				}
			});
		}
		return vieweModeButton;
	}
	/**
	 * This method initializes jButton2	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	private JButton getStopButton() {
		if (stopButton == null) {
			stopButton = new JButton();
			stopButton.setText("Stop");
			stopButton.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12));
			stopButton.addActionListener(new java.awt.event.ActionListener() { 
				public void actionPerformed(java.awt.event.ActionEvent e) {    
					// The acquisition end of the GPS information from a satellite.
					serialPortAddapter.close();
					
					// The reception end which is information from other terminals.
					msgReceiver.close();
					
					// enabled start button.
					startButton.setEnabled(true);
					//unenabled stop button.
					stopButton.setEnabled(false);
				}
			});
		}
		return stopButton;
	}
	/**
	 * This method initializes jComboBox	
	 * 	
	 * @return javax.swing.JComboBox	
	 */    
	private JComboBox getJComboBox() {
		if (jComboBox == null) {
			jComboBox = new JComboBox();
			jComboBox.setFont(new java.awt.Font("Dialog", java.awt.Font.PLAIN, 12));
			jComboBox.addItemListener(new java.awt.event.ItemListener() { 
				public void itemStateChanged(java.awt.event.ItemEvent e) { 
					// change view mode.
					if (jComboBox.getSelectedItem() == viewMode[0]) {
						Viewer.getInstance().setViewMode(ViewerUtilities.NORMAL_VIEW);
					} else if (jComboBox.getSelectedItem() == viewMode[1]) {
						Viewer.getInstance().setViewMode(ViewerUtilities.ROCK_VIEW);
					} else if (jComboBox.getSelectedItem() == viewMode[2]) {
						Viewer.getInstance().setViewMode(ViewerUtilities.OWN_VIEW);
					}
				}
			});
		}
		// adding item..
		for (int i = 0; i < viewMode.length; i++) {
			jComboBox.addItem(viewMode[i]);
		}
		return jComboBox;
	}
	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel() {
		if (jPanel == null) {
			GridLayout gridLayout1 = new GridLayout();
			jPanel = new JPanel();
			jPanel.setLayout(gridLayout1);
			gridLayout1.setRows(1);
			jPanel.add(getStartButton(), null);
			jPanel.add(getStopButton(), null);
		}
		return jPanel;
	}
	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */    
	private JPanel getJPanel1() {
		if (jPanel1 == null) {
			GridLayout gridLayout21 = new GridLayout();
			jPanel1 = new JPanel();
			jPanel1.setLayout(gridLayout21);
			gridLayout21.setRows(2);
			jPanel1.add(getVieweModeButton(), null);
			jPanel1.add(getJComboBox(), null);
		}
		return jPanel1;
	}
      }
