/*
 * Created on 2005/01/22
 *
 */
package jnet.hiromicam.algorithm.event;

import java.util.ArrayList;

import jnet.hiromicam.msg.FirstLevelInfo;
import jnet.hiromicam.properity.AlgorithmProperty;

/**
 * This class is algorithm for analyzing First-Level Information.<br>
 * This thgorithm use following function to detect abnormal action of this terminal.<br><br>
 * 
 * S(current) > S(average) + alpha * sigma(S)
 * <br><br>
 * S(current) is change of speed between current speed and 1 sec befor speed.<br>
 * S(average) is averate of change of speed. This value is gaved by as follow.<br>
 * S(average) = S(1) + S(2) + ... + S(5)<br>
 * S(n) is change of speed between n sec befor speed and (n - 1) sec befor speed<br>
 * sigma(S) is standard deviation of change of speed.<br>
 * alpha is coefficient.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class SimpleSpeedActionAnalyzer implements ActionAnalyzer {
	/**
	 * The number of using analyzed data.
	 */
	private static final int NUM = 5;
	
	/**
	 * This method analyze First-Level Information of won terminal.<br>
	 * If This terminal's action is abnormal, return true.<br><br>
	 * 
	 * 
	 * @param firstLevelInfoList First-Level Information of own.
	 * @return This terminal is abnormal or not. true:abnormal, false:normal.
	 * @see jnet.hiromicam.algorithm.event.ActionAnalyzer#isAbnormalAction(FirstLevelInfo, java.util.ArrayList)
	 */
	public boolean isAbnormalAction(FirstLevelInfo msg, ArrayList firstLevelInfoList) {
		
		double[] speed = new double[NUM];		// speed of each n time befor.
		double[] cs = new double[NUM - 1];		// change of speed of each n time befor.
		double ave;								// average change of speed.
		double sigma;							// standard deviation of change of speed.
		
		if (firstLevelInfoList.size() >= 5 + 1) {
			// set speed to array from list.
			for (int i = 0; i < NUM; i++) {
				// get speed of i sec befor
				speed[i] = Double.parseDouble(
								((FirstLevelInfo)firstLevelInfoList.get(firstLevelInfoList.size() - 1 - i)).getSpeed());
			}
			
			// set change of speed to array
			double sum = 0;
			for (int i = 0; i < NUM - 1; i++) {
				cs[i] = Math.abs(speed[i] - speed[i + 1]);
				sum += cs[i];
			}
			ave = sum / (NUM - 1);
			
			// set distribution and standard deviation.
			sum = 0;
			for (int i = 0; i < NUM - 1; i++) {
				sum += (ave - cs[i]) * (ave - cs[i]);
			}
			sigma = Math.sqrt(sum / (NUM - 1));
			
			double s = Math.abs(Double.parseDouble(msg.getSpeed()) - cs[0]);
			if ( s > ave + AlgorithmProperty.getInstance().getAlpha() * sigma) {
				// S(current) > S(average) + alpha * sigma(S)
				return true;
			}
		}
		
		// if data in list is less than 5, this sytem don't analyze.
		return false;
	}

}
