/*
 * Created on 2005/01/23
 *
 */
package jnet.hiromicam.buff;

import java.util.ArrayList;

import jnet.hiromicam.msg.AbstractSecondLevelInfo;

/**
 * This class for logging Second-Level Infomation.
 * As for this class, only one exists in this program.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class SecondLevelInfoBuff {
	/**
	 * This is Buffer size of this buffer.
	 */
	private static final int BUFF_SIZE = 50;
	
	/**
	 * This is only instance of SecondLevelInfoBuff.
	 */
	private static SecondLevelInfoBuff secondLevelInfoBuff = new SecondLevelInfoBuff();
	
	/**
	 *  List of imcoming Second-Level Infomation.
	 */
	private ArrayList secondLevelInfoList = new ArrayList();
	
	/**
	 * nobady create instance from constractor.
	 *
	 */
	private SecondLevelInfoBuff() {
		
	}
	
	/**
	 * This method reuturn instance of this class.
	 * 
	 * @return instance of this class.
	 */
	public static SecondLevelInfoBuff getInstance() {
		return secondLevelInfoBuff;
	}
	
	/**
	 * This method return Buffer size
	 * 
	 * @return Returns the BUFF_SIZE.
	 */
	public static int getBUFF_SIZE() {
		return BUFF_SIZE;
	}
	
	/**
	 * This method return Second-Level Infomation list.
	 * 
	 * @return Returns the lowLevelInfoList.
	 */
	public ArrayList getSecondLevelInfoList() {
		return secondLevelInfoList;
	}
	
	/**
	 * This method is for adding Second-Level Information to log.
	 * 
	 * @param msg the Second-Level Information to log.
	 * @return true : success adding message, false : other
	 */
	public boolean addSecondLevelInfo(AbstractSecondLevelInfo msg) {
		
		for (int i = 0; i < secondLevelInfoList.size(); i++) {
			AbstractSecondLevelInfo targetMsg = 
					(AbstractSecondLevelInfo)secondLevelInfoList.get(i);
			// If this terminal have the content that has same content ID,
			// this system don't log received information.
			if (msg.getContentID().equals(targetMsg.getContentID())) {
				return false;
			}
		}
		
		secondLevelInfoList.add(msg);
		return true;
	}
	
	// This method override javadoc of super class.
	public String toString() {
		String str = "";
		
		for (int i = 0; i < secondLevelInfoList.size(); i++) {
			str += ((AbstractSecondLevelInfo)secondLevelInfoList.get(i)).getMsg() + "\n";
		}
		
		return str;
	}
}
