/*
 * Created on 2005/01/21
 *
 */
package jnet.hiromicam.core;

import java.io.IOException;
import java.net.DatagramPacket;
import java.net.DatagramSocket;

import jnet.hiromicam.buff.FirstLevelInfoBuff;
import jnet.hiromicam.buff.SecondLevelInfoBuff;
import jnet.hiromicam.msg.AbstractSecondLevelInfo;
import jnet.hiromicam.msg.FirstLevelInfo;
import jnet.hiromicam.msg.Msg;
import jnet.hiromicam.msg.MsgTypeException;
import jnet.hiromicam.msg.pase.FirstLevelInfoPaser;
import jnet.hiromicam.msg.pase.SecondLevelInfoPaser;
import jnet.hiromicam.properity.SystemProperty;

/**
 * This class is for receiving process.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class MsgReceiver implements Runnable {
	/**
	 * Max Packet Size
	 */
	private final int MAX_PACKET_SIZE = 10240;
	
	/**
	 * true:start receive, false:stop receive
	 */
	private boolean receivigProcess = false;
	
	/**
	 * Paser
	 */
	private FirstLevelInfoPaser firstLevelPaser = new FirstLevelInfoPaser();
	
	/**
	 * Paser
	 */
	private SecondLevelInfoPaser secondLevelInfoPaser = new SecondLevelInfoPaser();
	
	/**
	 * This is a method for starting a receiving process.
	 * If information is received, it will record on a log and GpsViewer will be updated.
	 * 
	 * @see java.lang.Runnable#run()
	 * @throws when message type is wrong, it throws MsgTypeException.
	 */
	public void run() {
		try {
			// UDP socket for receiving
			
			DatagramSocket sock = new DatagramSocket(SystemProperty.PORT);	
			
			// UDP packet for receiving
			DatagramPacket packet =
					new DatagramPacket(new byte[MAX_PACKET_SIZE], MAX_PACKET_SIZE);
			
			// start receiving process.
			receivigProcess = true;		// true:start receive, false:stop receive
			while (receivigProcess) {
				sock.receive(packet);	// receiving packet from other terminal.
				
				// UDP Packet that this terminal sent is eliminated.
				if (!SystemProperty.getInstance().getAddr().equals(packet.getAddress().getHostAddress())) {
					// get data in string from reveived packet.
					String xmlMsg = new String(packet.getData(), 0, packet.getLength());
					
					// analyze message type.
					if (xmlMsg.endsWith("</" + Msg.FIRST_LEVEL_INFO_TAG + ">")) {
						// when received message is First-Level Information.
						Msg msg = firstLevelPaser.paseXML(xmlMsg);
						
						// Logging to Logger.
						FirstLevelInfoBuff.getInstance().addFirstLevelInfo(((FirstLevelInfo)msg).getIpAddr(),
																			((FirstLevelInfo)msg).getHostName(),
																			(FirstLevelInfo)msg);
						
						// deisplay received information
						System.out.println(msg.getMsg());
						
					} else if (xmlMsg.endsWith("</" + Msg.SECOND_LEVEL_INFO_TAG + ">")) {
						// When received message is Second-Level Information.
						Msg msg = secondLevelInfoPaser.paseXML(xmlMsg);
						
						// Logging to SecondLevelBuff
						SecondLevelInfoBuff.getInstance().addSecondLevelInfo((AbstractSecondLevelInfo)msg);
						
						// display received information
						System.out.println(msg.getMsg());
						
					} else {
						throw new MsgTypeException();
					}
				}
				
				// reset packet size.
				packet.setLength(MAX_PACKET_SIZE);
			}
			
			sock.close();		// close socket.
			
		} catch (IOException e) {
			System.out.println("Error : IOException.");
			e.printStackTrace();
		} catch (MsgTypeException e) {
			System.out.println("Error : MsgTypeException.");
			e.printStackTrace();
		}
	}
	
	/**
	 * The reception end which is information from other terminals.
	 *
	 */
	public void start() {
		new Thread(this, "ReceivingProcess").start();
	}
	
	/**
	 * A receiving process is ended by this method.
	 *
	 */
	public void close() {
		receivigProcess = false;
		System.out.println("close UDP port for receiving Hiromicam Packet.");
	}
}
