/*
 * Created on 2005/01/29
 *
 */
package jnet.hiromicam.msg;

import java.util.ArrayList;

/**
 * This is a class for Emergencies Information. 
 * This information is generated from the data processing of Low-Level Information or given by users. 
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class EmergencyInfo extends AbstractSecondLevelInfo {
	
	/**
	 * This represents an attribute of this message type.
	 */
	public static final String EMERGENCY_ATTRIBUTE = "Emergency";
	
	/**
	 * This represents tag of the accident time.
	 */
	public static final String ACCIDENT_TIME_TAG = "AccidentTime";
	
	/**
	 * This represents tag of the accident latitude.
	 */
	public static final String ACCIDENT_LATITUDE_TAG = "AccidentLatitude";
	
	/**
	 * This represents tag of the accident longitude.
	 */
	public static final String ACCIDENT_LONGITUDE_TAG = "AccidentLongitude";
	
	/**
	 * This represents tag of the accident type.
	 */
	public static final String ACCIDENT_TYPE_TAG_TAG = "AccidentType";
	
	/**
	 * This represents tag of the message.
	 */
	public static final String DETAIL_INFO_TAG = "Detail_Infomation";
	
	/**
	 * the time when an accident occured.
	 */
	protected String accidentTime;
	
	/**
	 * the latitude where an accident occured.
	 */
	protected String accidentLatitude;
	
	/**
	 * the longitude where an accident occured.
	 */
	protected String accidentLongitude;
	
	/**
	 * the type of the accident. For example, a fire, an explostion accident, and so on.
	 */
	protected String accidentType;
	
	/**
	 * the information of this emergency situation.
	 */
	protected String detailInfo;
	
	/**
	 * This is a default constractor.
	 * 
	 * @param contentID the content ID of this content.
	 * @param generatedTime the time when this content was generated.
	 * @param generator the IP Address who generated this content.
	 * @param sourceContentID the source content ID of this content.
	 * @param accidentTime the time when an accident occured.
	 * @param accidentLatitude the latitude where an accident occured.
	 * @param accidentLongitude the longitude where an accident occured.
	 * @param accidentType the type of the accident.
	 * @param msg the information of this emergency situation.
	 */
	public EmergencyInfo(String contentID,
						  String generatedTime,
						  String generator,
						  ArrayList sourceContentID,
						  String accidentTime,
						  String accidentLatitude,
						  String accidentLongitude,
						  String accidentType,
						  String detailInfo) {
		super(contentID, EMERGENCY_ATTRIBUTE, PRIORITY_HIGH, generatedTime, generator, sourceContentID);
		this.accidentTime = accidentTime;
		this.accidentLatitude = accidentLongitude;
		this.accidentLongitude = accidentLongitude;
		this.accidentType = accidentType;
		this.detailInfo = detailInfo;
	}
	
	/**
	 * This is a constractor for generating new content.
	 * 
	 * @param sourceContentID the source content ID of this content.
	 * @param accidentTime the time when an accident occured.
	 * @param accidentLatitude the latitude where an accident occured.
	 * @param accidentLongitude the longitude where an accident occured.
	 * @param accidentType the type of the accident.
	 * @param msg the information of this emergency situation.
	 */
	public EmergencyInfo(ArrayList sourceContentID,
						  String accidentTime,
						  String accidentLatitude,
						  String accidentLongitude,
						  String accidentType,
						  String detailInfo) {
		super(EMERGENCY_ATTRIBUTE, PRIORITY_HIGH, sourceContentID);
		this.accidentTime = accidentTime;
		this.accidentLatitude = accidentLongitude;
		this.accidentLongitude = accidentLongitude;
		this.accidentType = accidentType;
		this.detailInfo = detailInfo;
	}
	
	/**
	 * This method returns the latitude where the accident occured.
	 * 
	 * @return Returns the accidentLatitude.
	 */
	public String getAccidentLatitude() {
		return accidentLatitude;
	}
	
	/**
	 * This method returns the longitude where the accident occured.
	 * 
	 * @return Returns the accidentLongitude.
	 */
	public String getAccidentLongitude() {
		return accidentLongitude;
	}
	
	/**
	 * This method returns the time when the accident occured.
	 * 
	 * @return Returns the accidentTime.
	 */
	public String getAccidentTime() {
		return accidentTime;
	}
	
	/**
	 * This method returns the accident type.
	 * 
	 * @return Returns the accidentType.
	 */
	public String getAccidentType() {
		return accidentType;
	}
	
	/**
	 * This method returns the detail information of this accident.
	 * 
	 * @return Returns the detailInfo.
	 */
	public String getDetailInfo() {
		return detailInfo;
	}
	
	/* (non-Javadoc)
	 * @see jnet.hiromicam.msg.AbstractSecondLevelInfo#getConcreatMsg()
	 */
	public String getConcreatMsg() {
		return "\t<" + CONTENT_TAG + ">\n"
				+ "\t\t<" + ACCIDENT_TIME_TAG + ">" + accidentTime + "</" + ACCIDENT_TIME_TAG + ">\n"
				+ "\t\t<" + ACCIDENT_LATITUDE_TAG + ">" + accidentLatitude + "</" + ACCIDENT_LATITUDE_TAG + ">\n"
				+ "\t\t<" + ACCIDENT_LONGITUDE_TAG + ">" + accidentLongitude + "</" + ACCIDENT_LONGITUDE_TAG + ">\n"
				+ "\t\t<" + ACCIDENT_TYPE_TAG_TAG + ">" + accidentType + "</" + ACCIDENT_TYPE_TAG_TAG + ">\n"
				+ "\t\t<" + DETAIL_INFO_TAG + ">" + detailInfo + "</" + DETAIL_INFO_TAG + ">\n"
				+ "\t</" + CONTENT_TAG + ">\n";
	}

}
