/*
 * Created on 2005/01/20
 *
 */
package jnet.hiromicam.msg.factory;

import java.util.StringTokenizer;

import jnet.gps.process.DirectionCalculator;
import jnet.gps.process.SpeedCalculator;
import jnet.hiromicam.msg.FirstLevelInfo;

/**
 * This is factory class for First-Level Information.
 * 
 * @author Kazuya Sakai
 * @version 1.0
 * @since jdk 1.4.2
 */
public class FirstLevelInfoFactory {
	/**
	 * algorithm for calculating speed.
	 */
	private SpeedCalculator speedCalculator;
	
	/**
	 * algorithm for headding calculator.
	 */
	private DirectionCalculator directionCalculator;
	
	/**
	 * latitude of 1 sec befor.
	 */
	private String exLatitude;
	
	/**
	 * longitude of 1 sec befor.
	 */
	private String exLongitude;
	
	/**
	 * This is a default constractor.
	 * 
	 * @param speedCalculator algorithm for calculating speed
	 * @param directionCalculator algorithm for headding calculator
	 */
	public FirstLevelInfoFactory(SpeedCalculator speedCalculator,
								 DirectionCalculator directionCalculator) {
		this.speedCalculator = speedCalculator;
		this.directionCalculator = directionCalculator;
	}
	
	/**
	 * This method is changed into the format used by Hiromicam network from a GGA format.
	 * 
	 * String of GGA format<br>
	 * GGA - Global Positioning System Fix Data<br>
	 * GGA,123519.00,4807.038247,N,01131.324523,E,1,08,0.9,545.42,M,46.93,M,5.0,1012*42<br>
	 * 
	 * 1 : time<br>
	 * 2 : latitude<br>
	 * 4 : longitude<br>
	 * 5 : position state<br>
	 * 6 : satellite<br>
	 * 11: altitude<br>
	 * 
	 * @param ggaData GGA date.
	 * @return GpsInfo GPS infomation.
	 */
	public FirstLevelInfo createFirstLevelInfo(String ggaData) {
		
		String time = null;			// time
		String latitude = null;		// latitude
		String longitude = null;	// longitude
		String measurement = null; 	// measurement state
		String satellite = null;	// the number of satellite
		String altitude = null;		// altitude
		String speed = null;		// speed
		String direction = null;	// direction
		
		StringTokenizer st = new StringTokenizer(ggaData, ",");
		
		// sampling time, latitude and longitude from GGA format.
		for(int i = 0; st.hasMoreTokens() == true; i++) {
			if(i == 1) {
				time = st.nextToken(); 			// time
			} else if (i == 2) {
				latitude = st.nextToken();		// latitude
			} else if (i == 4) {
				longitude = st.nextToken();		// longitude
			} else if (i == 6) {
				measurement = st.nextToken();	// measurement stete
			} else if (i == 7) {
				satellite = st.nextToken();		// the number of satellite
			} else if (i == 11) {
				altitude = st.nextToken();		// altitude
			} else {
				st.nextToken();
			}
		}
		
		if (latitude != null && longitude != null 
				&& exLatitude != null && exLongitude != null) {
		// calculating speed.
		speed = speedCalculator.getSpeed(latitude, longitude, exLatitude, exLongitude);
		
		// calculating direction.
		direction = directionCalculator.getDirection(latitude, longitude, exLatitude, exLongitude);
		}
		
		// log current location.
		exLatitude = latitude;
		exLongitude = longitude;
		
		return new FirstLevelInfo(time,
								  latitude,
								  longitude,
								  measurement,
								  satellite,
								  altitude,
								  speed,
								  direction);
	}
	
}
